From 1ea85e6448ecaffbea1efd95c67b2e3fe3293476 Mon Sep 17 00:00:00 2001
From: Pierre d'Herbemont <pdherbemont@free.fr>
Date: Tue, 23 Feb 2010 15:28:06 +0100
Subject: [PATCH] sd: Add a new API for indicating if the SD is currently gathering data.

---
 include/vlc_events.h              |    7 ++++-
 include/vlc_services_discovery.h  |    3 ++
 modules/misc/lua/libs/sd.c        |    9 ++++++
 src/playlist/services_discovery.c |   51 +++++++++++++++++++++++++++++++++++++
 4 files changed, 69 insertions(+), 1 deletions(-)

diff --git a/include/vlc_events.h b/include/vlc_events.h
index 50cbc61..5cd21d6 100644
--- a/include/vlc_events.h
+++ b/include/vlc_events.h
@@ -130,7 +130,8 @@ typedef enum vlc_event_type_t {
     vlc_ServicesDiscoveryItemAdded,
     vlc_ServicesDiscoveryItemRemoved,
     vlc_ServicesDiscoveryStarted,
-    vlc_ServicesDiscoveryEnded
+    vlc_ServicesDiscoveryEnded,
+    vlc_ServicesDiscoveryLoadingChanged
 } vlc_event_type_t;
 
 /* Event definition */
@@ -202,6 +203,10 @@ typedef struct vlc_event_t
         {
             void * unused;
         } services_discovery_ended;
+        struct services_discovery_loading_changed
+        {
+            bool new_value;
+        } services_discovery_loading_changed;
 
     } u;
 } vlc_event_t;
diff --git a/include/vlc_services_discovery.h b/include/vlc_services_discovery.h
index 956b77b..609b605 100644
--- a/include/vlc_services_discovery.h
+++ b/include/vlc_services_discovery.h
@@ -96,6 +96,9 @@ VLC_EXPORT( vlc_event_manager_t *,  services_discovery_EventManager, ( services_
 VLC_EXPORT( void,                   services_discovery_AddItem, ( services_discovery_t * p_this, input_item_t * p_item, const char * psz_category ) );
 VLC_EXPORT( void,                   services_discovery_RemoveItem, ( services_discovery_t * p_this, input_item_t * p_item ) );
 
+/* Used by services_discover module to hint to the client that it is currently loading */
+VLC_EXPORT( void,                   services_discovery_SetLoading, ( services_discovery_t * p_this, bool loading ) );
+
 
 /* SD probing */
 
diff --git a/modules/misc/lua/libs/sd.c b/modules/misc/lua/libs/sd.c
index a5bb09f..5ab0204 100644
--- a/modules/misc/lua/libs/sd.c
+++ b/modules/misc/lua/libs/sd.c
@@ -314,6 +314,14 @@ static int vlclua_node_add_node( lua_State *L )
     return 1;
 }
 
+static int vlclua_sd_set_loading( lua_State *L )
+{
+    services_discovery_t *p_sd = (services_discovery_t *)vlclua_get_this( L );
+    bool isLoading = luaL_checkbool( L, 1 );
+    services_discovery_SetLoading( p_sd, isLoading );
+    return 1;
+}
+
 /*****************************************************************************
  *
  *****************************************************************************/
@@ -325,6 +333,7 @@ static const luaL_Reg vlclua_sd_reg[] = {
     { "add_node", vlclua_sd_add_node },
     { "add_item", vlclua_sd_add_item },
     { "remove_item", vlclua_sd_remove_item },
+    { "set_loading", vlclua_sd_set_loading },
     { NULL, NULL }
 };
 
diff --git a/src/playlist/services_discovery.c b/src/playlist/services_discovery.c
index 2afb024..8ec0799 100644
--- a/src/playlist/services_discovery.c
+++ b/src/playlist/services_discovery.c
@@ -122,6 +122,7 @@ services_discovery_t *vlc_sd_Create( vlc_object_t *p_super,
     vlc_event_manager_register_event_type(em, vlc_ServicesDiscoveryItemRemoved);
     vlc_event_manager_register_event_type(em, vlc_ServicesDiscoveryStarted);
     vlc_event_manager_register_event_type(em, vlc_ServicesDiscoveryEnded);
+    vlc_event_manager_register_event_type(em, vlc_ServicesDiscoveryLoadingChanged);
 
     vlc_object_set_destructor( p_sd, services_discovery_Destructor );
     vlc_object_attach( p_sd, p_super );
@@ -229,6 +230,19 @@ services_discovery_RemoveItem ( services_discovery_t * p_sd, input_item_t * p_it
     vlc_event_send( &p_sd->event_manager, &event );
 }
 
+/***********************************************************************
+ * RemoveItem
+ ***********************************************************************/
+void
+services_discovery_SetLoading ( services_discovery_t * p_sd, bool loading )
+{
+    vlc_event_t event;
+    event.type = vlc_ServicesDiscoveryLoadingChanged;
+    event.u.services_discovery_loading_changed.new_value = loading;
+
+    vlc_event_send( &p_sd->event_manager, &event );
+}
+
 /*
  * Playlist - Services discovery bridge
  */
@@ -237,6 +251,7 @@ struct vlc_sd_internal_t
 {
     /* the playlist items for category and onelevel */
     playlist_item_t      *p_node;
+    playlist_item_t      *p_loading_item;
     services_discovery_t *p_sd; /**< Loaded service discovery modules */
     char                 *psz_name;
 };
@@ -300,6 +315,39 @@ static void playlist_sd_item_removed( const vlc_event_t * p_event, void * user_d
     PL_UNLOCK;
 }
 
+/* A certain sd is loading */
+static void playlist_sd_loading_changed( const vlc_event_t *p_event, void *user_data )
+{
+    bool isLoading = p_event->u.services_discovery_loading_changed.new_value;
+    vlc_sd_internal_t *p_sds = user_data;
+    playlist_t *p_playlist = p_sds->p_node->p_playlist;
+
+    if( !isLoading )
+    {
+        PL_LOCK;
+        if( p_sds->p_loading_item )
+        {
+            /* Remove the sd loading node if it exists */
+            playlist_NodeDelete( p_playlist, p_sds->p_node, true, false );
+            p_sds->p_loading_item = NULL;
+        }
+        PL_UNLOCK;
+    }
+    else
+    {
+        PL_LOCK;
+        if( !p_sds->p_loading_item )
+        {
+            /* Add a node for "Loading..." */
+            p_sds->p_loading_item = playlist_NodeCreate( p_playlist,
+                                                        _("Loading..."),
+                                                        p_sds->p_node,
+                                                        0, NULL );
+        }
+        PL_UNLOCK;
+    }
+}
+
 int playlist_ServicesDiscoveryAdd( playlist_t *p_playlist,
                                    const char *psz_name )
 {
@@ -349,6 +397,9 @@ int playlist_ServicesDiscoveryAdd( playlist_t *p_playlist,
     vlc_event_attach( em, vlc_ServicesDiscoveryItemRemoved,
                       playlist_sd_item_removed, p_node );
 
+    vlc_event_attach( em, vlc_ServicesDiscoveryLoadingChanged,
+                     playlist_sd_loading_changed, p_sds );
+
     if( !vlc_sd_Start( p_sd ) )
     {
         vlc_sd_Destroy( p_sd );
-- 
1.7.0

